#!/bin/sh

# here is readahead in KB that we use
# this seems huge but for a modern RAID volume that can do 1 GB/s,
# this is just 65 msec of data, enough to optimize away some seeks on
# a multi-stream workload but not enough to kill response time

tuned_ra=65536
default_ra=128
tuned_nrq=512
default_nrq=128

. /etc/tune-profiles/functions

# set readahead and nr_requests in first & second CLI parameters on ALL
# multipath devices


set_multipath_ra_nrq() {
  ra=$1
  nrq=$2
  # don't do this unless there is multipath CLI installed on system
  which multipath > /tmp/mp.tmp 2>&1 || return
  multipath_devs=`multipath -l | grep -- ' dm-' | awk '{ print $1 }'`
  echo
  echo -n "setting readahead to $ra and nr_requests to $nrq on multipath devices: "
  for n in $multipath_devs ; do
    device_mapper_path=`readlink /dev/mapper/$n`
    device_mapper_name=`basename $device_mapper_path`
    echo -n " $device_mapper_name"
    echo $ra > /sys/block/$device_mapper_name/queue/read_ahead_kb
    echo $nrq > /sys/block/$device_mapper_name/queue/nr_requests
  done
  echo
}

start() {
	set_cpu_governor performance
	enable_transparent_hugepages

	# Find non-root and non-boot partitions, disable barriers on them
	rootvol=$(df -h / | grep "^/dev" | awk '{print $1}')
	bootvol=$(df -h /boot | grep "^/dev" | awk '{print $1}')
	volumes=$(df -hl --exclude=tmpfs | grep "^/dev" | awk '{print $1}')

	nobarriervols=$(echo "$volumes" | grep -v $rootvol | grep -v $bootvol)
	remount_partitions nobarrier $nobarriervols
	for d in $DISKS ; do 
		blkdev=`basename $d`
		echo 65536 > /sys/block/$blkdev/queue/read_ahead_kb
		echo 512 > /sys/block/$blkdev/queue/nr_requests 
	done
        set_multipath_ra_nrq $tuned_ra $tuned_nrq
	return 0
}

stop() {

	# Find non-root and non-boot partitions, re-enable barriers
	rootvol=$(df -h / | grep "^/dev" | awk '{print $1}')
	bootvol=$(df -h /boot | grep "^/dev" | awk '{print $1}')
	volumes=$(df -hl --exclude=tmpfs | grep "^/dev" | awk '{print $1}')

	nobarriervols=$(echo "$volumes" | grep -v $rootvol | grep -v $bootvol)
	remount_partitions barrier $nobarriervols
	for d in $DISKS ; do 
		blkdev=`basename $d`
		echo 128 > /sys/block/$blkdev/queue/read_ahead_kb
		echo 128 > /sys/block/$blkdev/queue/nr_requests
	done
        set_multipath_ra_nrq $default_ra $default_nrq
	restore_cpu_governor
	restore_transparent_hugepages
	return 0
}

process $@
